;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Ram Viewer, version 0.2
;
;This is the code to a hex viewer.  At this moment, it is almost a
;copy of hex editor, except for the keys and the numbers are not very
;tiny.
;
;By Kevin Panko, bunnyman1@juno.com
;If you use this, and you have e-mail access, you must e-mail my address.
;You will then be registered, and God will help you find happiness.
;I will also send you the newest version.  (UUencoded, I can't attach files.)
;Note: last week I got geocities' e-mail, so I've got mime mail (for the
;moment.)

;To do list:  Add ram editing functionality.
;             Allow user to jump to an address.


#include "ti-85.h"

.org 0
.db "Ram Viewer (F1 is help)", 0


;variables
curptr  =TEXT_MEM             ;points to wherever we are in the memory
curmode =curptr+2             ;bit 0 on means alpha mode is on
alphamode =0                  ;when set, show ascii in right column


main:
  ROM_CALL(CLEARLCD)          ;clear screen

  xor a
  ld (curptr), a              ;zero the pointer
  CALL_(drawscreen)           ;draw the screen


keyloop:                      ;wait for key
  call GET_KEY                ;once key is pushed, jump to one of the below
  cp K_EXIT                   ;if no key pushed, loop and check again.
  ret z
  cp K_UP
  jr z, subeight
  cp K_DOWN
  jr z, addeight
  cp K_CLEAR
  jr z, subhundred
  cp K_RAISE
  jr z, addhundred
  cp K_CUSTOM
  jr z, subthousand
  cp K_TAN
  jr z, addthousand
  cp K_ALPHA
  CALL_Z(alphakey)
  cp K_F1
  CALL_Z(helpscreen)
  jr keyloop

subeight:                     ;this group will change the curptr variable
  ld hl, -8h
  ld a, (curmode)             ;in alpha mode, one line is only 4 bytes
  bit alphamode, a
  jr z, advanceptr
  ld hl, -4h
  jr advanceptr
addeight:
  ld hl, 8h
  ld a, (curmode)             ;in alpha mode, one line is only 4 bytes
  bit alphamode, a
  jr z, advanceptr
  ld hl, 4h
  jr advanceptr
subhundred:
  ld hl, -100h
  jr advanceptr
addhundred:
  ld hl, 100h
  jr advanceptr
subthousand:
  ld hl, -1000h
  jr advanceptr
addthousand:
  ld hl, 1000h
advanceptr:
  ld de, (curptr)
  add hl, de
  ld (curptr),  hl
  CALL_(drawscreen)
  jr keyloop


drawscreen:                   ;draws the screen using curptr as the top
  push af
  push bc
  push hl
  xor a                       ;zero a
  ld (CURSOR_ROW2), a         ;home the cursor
  ld (CURSOR_COL2), a

  ld hl, (curptr)

  ld b, 8                     ;for each line (there are 8)
nextline:
  CALL_(drawline)             ;draw the line
  djnz nextline

  pop hl
  pop bc
  pop af
  ret



drawline:                     ;this will draw one line of hex
  push af
  push bc

  ld a, h                     ;print the address first
  CALL_(printhex)
  ld a, l
  CALL_(printhex)

  ld a, ' '                   ;leave a space
  CALL_(putchar)

  push hl                     ;save hl in case we need the ptr later
  ld b, 4                     ;show 4 bytes of memory on the line
nextbytepart1:
  ld a, (hl)
  inc hl
  CALL_(printhex)
  djnz nextbytepart1

  ld a, (curmode)
  bit alphamode, a
  jr nz, printalpha

  ld b, 4                     ;show 4 bytes of memory on the line
nextbytepart2:
  ld a, (hl)
  inc hl
  CALL_(printhex)
  djnz nextbytepart2
  pop bc                      ;discard this value
  pop bc
  pop af
  ret

printalpha:
  ld a, ' '
  CALL_(putchar)
  CALL_(putchar)

  pop hl                      ;this value is used here
  ld b, 4                     ;show 4 bytes of memory on the line
nextletter:
  ld a, (hl)
  inc hl
  CALL_(putchar)
  djnz nextletter

  ld a, ' '
  CALL_(putchar)
  CALL_(putchar)

  pop bc
  pop af
  ret


alphakey:                     ;will toggle the alphamode bit in curmode
  push af
  ld a, (curmode)
  bit alphamode, a            ;notice that this changes the zero flag
  set alphamode, a            ;while this affects no flags at all
  jr z, skipclearalpha
  res alphamode, a
skipclearalpha:
  ld (curmode), a
  CALL_(drawscreen)
  pop af
  ret


helpscreen:                 ;show the help screen, then the credits
  ROM_CALL(CLEARLCD)

  xor a
  ld (CURSOR_ROW2), a
  ld (CURSOR_COL2), a
  ld de, helpstring
  ld hl, (PROGRAM_ADDR)     
  add hl, de
  ROM_CALL(D_ZT_STR)        ;help screen
helpkeyloop1:               ;wait for key
  call GET_KEY
  or a
  jr z, helpkeyloop1

  ROM_CALL(CLEARLCD)

  xor a
  ld (CURSOR_ROW2), a
  ld (CURSOR_COL2), a
  ld de, creditstring1
  ld hl, (PROGRAM_ADDR)
  add hl, de
  ROM_CALL(D_ZT_STR)        ;credit screen
  CALL_(newline)
  CALL_(newline)
  ld de, creditstring2
  ld hl, (PROGRAM_ADDR)
  add hl, de
  ROM_CALL(D_ZT_STR)        ;part 2
  CALL_(newline)
  CALL_(newline)
  ld de, creditstring3
  ld hl, (PROGRAM_ADDR)     ;and 3
  add hl, de
  ROM_CALL(D_ZT_STR)
helpkeyloop2:               ;wait for key
  call GET_KEY
  or a
  jr z, helpkeyloop2

  CALL_(drawscreen)
  ret


printhex:            ;print A as hex (two characters)
  push af
  push bc

  ld b, a                     ;save A in B
  and $F0                     ;mask out the lower 4 bits
  rrca                        ;rotate right circular accumulator
  rrca
  rrca
  rrca
  add a, '0'
  cp 3Ah
  jr c, is_numeric1
  add a, 7h                   ;if not a digit, align this digit to a letter
is_numeric1:
  CALL_(putchar)

  ld a, b                     ;retrieve A from B
  and 0Fh                     ;mask out the higher 4 bits
  add a, '0'                  ;do it all again
  cp 3Ah
  jr c, is_numeric2
  add a, 7h
is_numeric2:
  CALL_(putchar)

  pop bc
  pop af
  ret

putchar:                      ;put a single character from A on screen
                              ;without translation, and without scrolling
  ROM_CALL(TR_CHARPUT)

  push af
  ld a, (CURSOR_COL2)
  cp 20
  jr z, end_of_line
  inc a
  ld (CURSOR_COL2), a
  pop af
  ret
end_of_line:
  CALL_(newline)
  pop af
  ret


newline:                      ;this puts the cursor at the beginning of
                              ;the next line
  push af
  ld a, (CURSOR_ROW2)         ;inc the Y
  inc a
  ld (CURSOR_ROW2), a
  xor a                       ;and zero the X
  ld (CURSOR_COL2), a
  pop af
  ret


;data begins here

helpstring:
      ;123456789012345678901
  .db "UP    -Go  by 1h    "  ;1
  .db "DOWN  -Go  by 1h    "  ;2
  .db "CLEAR -Go  by 100h  "  ;3
  .db "^     -Go  by 100h  "  ;4 
  .db "CUSTOM-Go  by 1000h "  ;5
  .db "TAN   -Go  by 1000h "  ;6
  .db "ALPHA -Show ASCII    "  ;7
  .db "All numbers are hex.", 0;8


creditstring1:
      ;123456789012345678901 
  .db "   Ram Viewer v0.2", 0  ;1
creditstring2:                 ;2
  .db "    By Kevin Panko   "  ;3
  .db "  bunnyman1@juno.com", 0;4
creditstring3:                 ;5
  .db "      Thanks to      "  ;6   
  .db "  Philip Montgomery  "  ;7
  .db " And the ZShell Team", 0;8             

.end

